@@ -75,7 +75,7 @@ static	utype	*firstups = NULL;
 
 	/* signal handling things */
 static	struct sigaction sa;
-static	sigset_t upsm_sigmask;
+static	sigset_t nut_upsmon_sigmask;
 
 #ifdef SHUT_RDWR
 #define	shutdown_how SHUT_RDWR
@@ -209,7 +209,8 @@ static int checkmaster(utype *ups)
 
 	/* this shouldn't happen (LOGIN checks it earlier) */
 	if ((ups->upsname == NULL) || (strlen(ups->upsname) == 0)) {
-		upslogx(LOG_ERR, "Set master on UPS [%s] failed: empty upsname");
+		upslogx(LOG_ERR, "Set master on UPS [%s] failed: empty upsname",
+			ups->sys);
 		return 0;
 	}
 
@@ -290,7 +291,8 @@ static int do_upsd_auth(utype *ups)
 
 	/* we require a upsname now */
 	if ((ups->upsname == NULL) || (strlen(ups->upsname) == 0)) {
-		upslogx(LOG_ERR, "Login to UPS [%s] failed: empty upsname");
+		upslogx(LOG_ERR, "Login to UPS [%s] failed: empty upsname",
+			ups->sys);
 		return 0;
 	}
 
@@ -1291,7 +1293,7 @@ static void loadconfig(void)
 				snprintfcat(errmsg, sizeof(errmsg), " %s", 
 					ctx.arglist[i]);
 
-			upslogx(LOG_WARNING, errmsg);
+			upslogx(LOG_WARNING, "%s", errmsg);
 		}
 	}
 
@@ -1376,8 +1378,8 @@ static void read_timeout(int sig)
 /* install handlers for a few signals */
 static void setup_signals(void)
 {
-	sigemptyset(&upsm_sigmask);
-	sa.sa_mask = upsm_sigmask;
+	sigemptyset(&nut_upsmon_sigmask);
+	sa.sa_mask = nut_upsmon_sigmask;
 	sa.sa_flags = 0;
 
 	sa.sa_handler = sigpipe;
@@ -1675,7 +1677,7 @@ static int check_pdflag(void)
 
 	if (ret == -1) {
 		upslogx(LOG_ERR, "POWERDOWNFLAG (%s) does not contain "
-			"the upsmon magic string");
+			"the upsmon magic string", powerdownflag);
 		return EXIT_FAILURE;
 	}
 
