@@ -428,7 +428,7 @@ void upsdrv_initinfo(void)
 		exit(EXIT_FAILURE);
 	}
 
-	dstate_setinfo("ups.mfr", "BELKIN", 0, 0);
+	dstate_setinfo("ups.mfr", "BELKIN");
 	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 
 	/* see what's out there */
