@@ -28,7 +28,7 @@
 #include <asm/types.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
-#include <sys/time.h>
+#include "timehead.h"
 
 #include "config.h"
 
