@@ -1,8 +1,8 @@
 /* newhidups.c - Driver for serial/USB HID UPS units
  * 
- * Copyright (C) 2003 - 2005
- *   Arnaud Quette <arnaud.quette@free.fr> && <arnaud.quette@mgeups.com>
- *   John Stamp <kinsayder@hotmail.com>
+ * Copyright (C)
+ *   2003-2005 Arnaud Quette <http://arnaud.quette.free.fr/contact.html>
+ *   2005 John Stamp <kinsayder@hotmail.com>
  *
  * This program is sponsored by MGE UPS SYSTEMS - opensource.mgeups.com
  *
@@ -25,43 +25,255 @@
 #include "libhid.h"
 #include "newhidups.h"
 
-/* include all known hid2nut lookup tables */
+/* include all known subdrivers */
 #include "mge-hid.h"
 #include "apc-hid.h"
+#include "belkin-hid.h"
 
-/* pointer to the good lookup tables */
-static hid_info_t *hid_ups;
-static models_name_t *model_names;
+/* master list of avaiable subdrivers */
+static subdriver_t *subdriver_list[] = {
+	&mge_subdriver,
+	&apc_subdriver,
+	&belkin_subdriver,
+	NULL
+};
+
+/* pointer to the active subdriver object (set in upsdrv_initups, then
+   constant) */
+static subdriver_t *subdriver;
 
 /* Global vars */
 static HIDDevice *hd;
-static MatchFlags flg;
-static int offdelay = DEFAULT_OFFDELAY;
-static int ondelay = DEFAULT_ONDELAY;
+static HIDDevice curDevice;
+static HIDDeviceMatcher_t *reopen_matcher = NULL;
+static HIDDeviceMatcher_t *regex_matcher = NULL;
 static int pollfreq = DEFAULT_POLLFREQ;
 static int ups_status = 0;
 static bool data_has_changed = FALSE; /* for SEMI_STATIC data polling */
 static time_t lastpoll; /* Timestamp the last polling */
+usb_dev_handle *udev;
 
 /* support functions */
-static int instcmd(const char *cmdname, const char *extradata);
-static int setvar(const char *varname, const char *val);
 static hid_info_t *find_nut_info(const char *varname);
+static hid_info_t *find_nut_info_valid(const char *varname);
 static hid_info_t *find_hid_info(const char *hidname);
 static char *hu_find_infoval(info_lkp_t *hid2info, long value);
-static char *get_model_name(char *iProduct, char *iModel);
 static void process_status_info(char *nutvalue);
 static void ups_status_set(void);
 static void identify_ups ();
 static bool hid_ups_walk(int mode);
 static void reconnect_ups(void);
 
+/* ---------------------------------------------------------------------- */
+/* data for ups.status processing */
+
+status_lkp_t status_info[] = {
+	/* map internal status strings to bit masks */
+	{ "online", STATUS_ONLINE },
+	{ "dischrg", STATUS_DISCHRG },
+	{ "chrg", STATUS_CHRG },
+	{ "lowbatt", STATUS_LOWBATT },
+	{ "overload", STATUS_OVERLOAD },
+	{ "replacebatt", STATUS_REPLACEBATT },
+	{ "shutdownimm", STATUS_SHUTDOWNIMM },
+	{ "trim", STATUS_TRIM },
+	{ "boost", STATUS_BOOST },
+	{ "bypass", STATUS_BYPASS },
+	{ "off", STATUS_OFF },
+	{ "overhead", STATUS_OVERHEAT },
+	{ "commfault", STATUS_COMMFAULT },
+	{ "depleted", STATUS_DEPLETED },
+	{ "timelimitexp", STATUS_TIMELIMITEXP },
+	{ "batterypres", STATUS_BATTERYPRES },
+	{ NULL, 0 },
+};
+
+/* ---------------------------------------------------------------------- */
+/* value lookup tables and generic lookup functions */
+
+/* Actual value lookup tables => should be fine for all Mfrs (TODO: validate it!) */
+
+/* the purpose of the following status conversions is to collect
+   information, not to interpret it. The function
+   process_status_info() remembers these values by updating the global
+   variable ups_status. Interpretation happens in ups_status_set,
+   where they are converted to standard NUT status strings. Notice
+   that the below conversions do not yield standard NUT status
+   strings; this in indicated being in lower-case characters. 
+
+   The reason to separate the collection of information from its
+   interpretation is that not each report received from the UPS may
+   contain all the status flags, so they must be stored
+   somewhere. Also, there can be more than one status flag triggering
+   a certain condition (e.g. a certain UPS might have variables
+   low_battery, shutdown_imminent, timelimit_exceeded, and each of
+   these would trigger the NUT status LB. But we have to ensure that
+   these variables don't unset each other, so they are remembered
+   separately)  */
+
+info_lkp_t online_info[] = {
+  { 1, "online", NULL },
+  { 0, "!online", NULL },
+  { 0, "NULL", NULL }
+};
+info_lkp_t discharging_info[] = {
+  { 1, "dischrg", NULL },
+  { 0, "!dischrg", NULL },
+  { 0, "NULL", NULL }
+};
+info_lkp_t charging_info[] = {
+  { 1, "chrg", NULL },
+  { 0, "!chrg", NULL },
+  { 0, "NULL", NULL }
+};
+info_lkp_t lowbatt_info[] = {
+  { 1, "lowbatt", NULL },
+  { 0, "!lowbatt", NULL },
+  { 0, "NULL", NULL }
+};
+info_lkp_t overload_info[] = {
+  { 1, "overload", NULL },
+  { 0, "!overload", NULL },
+  { 0, "NULL", NULL }
+};
+info_lkp_t replacebatt_info[] = {
+  { 1, "replacebatt", NULL },
+  { 0, "!replacebatt", NULL },
+  { 0, "NULL", NULL }
+};
+info_lkp_t shutdownimm_info[] = {
+  { 1, "shutdownimm", NULL },
+  { 0, "!shutdownimm", NULL },
+  { 0, "NULL", NULL }
+};
+info_lkp_t trim_info[] = {
+  { 1, "trim", NULL },
+  { 0, "!trim", NULL },
+  { 0, "NULL", NULL }
+};
+info_lkp_t boost_info[] = {
+  { 1, "boost", NULL },
+  { 0, "!boost", NULL },
+  { 0, "NULL", NULL }
+};
+/* FIXME: extend ups.status for BYPASS Manual/Automatic */
+info_lkp_t bypass_info[] = {
+  { 1, "bypass", NULL },
+  { 0, "!bypass", NULL },
+  { 0, "NULL", NULL }
+};
+/* note: this value is reverted (0=set, 1=not set). We report "being
+   off" rather than "being on", so that devices that don't implement
+   this variable are "on" by default */
+info_lkp_t off_info[] = {
+  { 0, "off", NULL },
+  { 1, "!off", NULL },
+  { 0, "NULL", NULL }
+};
+/* FIXME: add CAL */
+
+info_lkp_t test_write_info[] = {
+  { 0, "No test", NULL },
+  { 1, "Quick test", NULL },
+  { 2, "Deep test", NULL },
+  { 3, "Abort test", NULL },
+  { 0, "NULL", NULL }
+};
+info_lkp_t test_read_info[] = {
+  { 1, "Done and passed", NULL },
+  { 2, "Done and warning", NULL },
+  { 3, "Done and error", NULL },
+  { 4, "Aborted", NULL },
+  { 5, "In progress", NULL },
+  { 6, "No test initiated", NULL },
+  { 0, "NULL", NULL }
+};
+
+info_lkp_t beeper_info[] = {
+  { 1, "disabled", NULL },
+  { 2, "enabled", NULL },
+  { 3, "muted", NULL },
+  { 0, "NULL", NULL }
+};
+
+/* returns statically allocated string - must not use it again before
+   done with result! */
+static char *date_conversion_fun(long value) {
+  static char buf[20];
+  int year, month, day;
+
+  if (value == 0) {
+    return "not set";
+  }
+
+  year = 1980 + (value >> 9); /* negative value represents pre-1980 date */ 
+  month = (value >> 5) & 0x0f;
+  day = value & 0x1f;
+  
+  sprintf(buf, "%04d/%02d/%02d", year, month, day);
+  return buf;
+}
+
+info_lkp_t date_conversion[] = {
+  { 0, NULL, date_conversion_fun }
+};
+
+/* returns statically allocated string - must not use it again before
+   done with result! */
+static char *hex_conversion_fun(long value) {
+	static char buf[20];
+	
+	sprintf(buf, "%08lx", value);
+	return buf;
+}
+
+info_lkp_t hex_conversion[] = {
+	{ 0, NULL, hex_conversion_fun }
+};
+
+/* returns statically allocated string - must not use it again before
+   done with result! */
+static char *stringid_conversion_fun(long value) {
+	static char buf[20];
+	libusb_get_string(udev, value, buf);	
+	
+	return buf;
+}
+
+info_lkp_t stringid_conversion[] = {
+	{ 0, NULL, stringid_conversion_fun }
+};
+
+/* ---------------------------------------------
+ * subdriver matcher
+ * --------------------------------------------- */
+
+static int match_function_subdriver(HIDDevice *d, void *privdata) {
+	int i;
+
+	for (i=0; subdriver_list[i] != NULL; i++) {
+		if (subdriver_list[i]->claim(d)) {
+			return 1;
+		}
+	}
+	return 0;
+}
+
+static HIDDeviceMatcher_t subdriver_matcher_struct = {
+	match_function_subdriver,
+	NULL,
+	NULL,
+};
+static HIDDeviceMatcher_t *subdriver_matcher = &subdriver_matcher_struct;
+
 /* ---------------------------------------------
  * driver functions implementations
  * --------------------------------------------- */
 void upsdrv_shutdown(void)
 {
-	char delay[7];
+	int offdelay = DEFAULT_OFFDELAY;
+	int ondelay = DEFAULT_ONDELAY;
+	int r;
 
 	/* Retrieve user defined delay settings */
 	if ( getval(HU_VAR_ONDELAY) )
@@ -70,51 +282,30 @@ void upsdrv_shutdown(void)
 	if ( getval(HU_VAR_OFFDELAY) )
 		offdelay = atoi( getval(HU_VAR_OFFDELAY) );
 
-	/* Apply specific method(s) */
-	switch (hd->VendorID)
-	{
-		case APC:
-			/* FIXME: the data (or command) should appear in
-			 * the hid2nut table, so that it can be autodetected
-			 * upon startup, and then calable through setvar()
-			 * or instcmd(), ie below
-			 */
-
-			/* From apcupsd, usb.c/killpower() */
-			/* 1) APCBattCapBeforeStartup */
-			/* 2) BackUPS Pro => */
-	
-			/* Misc method B */
-			upsdebugx(2, "upsdrv_shutdown: APC ForceShutdown style shutdown.\n");
-			if (instcmd("shutdown.return", NULL) != STAT_INSTCMD_HANDLED)
-				upsdebugx(2, "ForceShutdown command failed");
-
-
-		/* Don't "break" as the general method might also be supported! */;
-		case MGE_UPS_SYSTEMS:
-		default:
-			/* 1) set DelayBeforeStartup */
-			sprintf(&delay[0], "%i", ondelay);
-			if (setvar("ups.delay.start", &delay[0])!= STAT_SET_HANDLED)
-				fatalx("Shutoff command failed (setting ondelay)");
-			
-			/* 2) set DelayBeforeShutdown */
-			sprintf(&delay[0], "%i", offdelay);
-			if (setvar("ups.delay.shutdown", &delay[0])!= STAT_SET_HANDLED)
-				fatalx("Shutoff command failed (setting offdelay)");
-		break;
+	/* enforce ondelay > offdelay */
+	if (ondelay <= offdelay) {
+		ondelay = offdelay + 1;
+		upsdebugx(2, "Ondelay must be greater than offdelay; setting ondelay = %d (offdelay = %d)", ondelay, offdelay);
 	}
+
+	/* Apply specific method */
+	r = subdriver->shutdown(ondelay, offdelay);
+
+	if (r == 0) {
+		fatalx("Shutdown failed.");
+	}
+	upsdebugx(2, "Shutdown command succeeded.");
 }
 
 /* process instant command and take action. */
-static int instcmd(const char *cmdname, const char *extradata)
+int instcmd(const char *cmdname, const char *extradata)
 {
 	hid_info_t *hidups_item;
 	
 	upsdebugx(2, "entering instcmd(%s, %s)\n", cmdname, extradata);
 
 	/* Retrieve and check netvar & item_path */	
-	hidups_item = find_nut_info(cmdname);
+	hidups_item = find_nut_info_valid(cmdname);
 	
 	/* Check validity of the found the item */
 	if (hidups_item == NULL || hidups_item->info_type == NULL ||
@@ -133,7 +324,7 @@ static int instcmd(const char *cmdname, const char *extradata)
 	}
 	
 	/* Actual variable setting */
-	if (HIDSetItemValue(hidups_item->hidpath, atol(hidups_item->dfl)))
+	if (HIDSetItemValue(udev, hidups_item->hidpath, atol(hidups_item->dfl), subdriver->utab))
 	{
 		upsdebugx(3, "SUCCEED\n");
 		/* Set the status so that SEMI_STATIC vars are polled */
@@ -148,7 +339,7 @@ static int instcmd(const char *cmdname, const char *extradata)
 }
 
 /* set r/w variable to a value. */
-static int setvar(const char *varname, const char *val)
+int setvar(const char *varname, const char *val)
 {
 	hid_info_t *hidups_item;
 	
@@ -189,7 +380,7 @@ static int setvar(const char *varname, const char *val)
 	}
 
 	/* Actual variable setting */
-	if (HIDSetItemValue(hidups_item->hidpath, atol(val)))
+	if (HIDSetItemValue(udev, hidups_item->hidpath, atol(val), subdriver->utab))
 	{
 		/* FIXME: GetValue(hidups_item->hidpath) to ensure success on non volatile */
 		upsdebugx(3, "SUCCEED\n");
@@ -226,7 +417,13 @@ void upsdrv_makevartable(void)
 		DEFAULT_POLLFREQ);
 	addvar(VAR_VALUE, HU_VAR_POLLFREQ, temp);
 	
-	/* FIXME: autodetection values (Mfr, Product, Index, ...) */
+	/* allow -x vendor=X, vendorid=X, product=X, productid=X, serial=X */
+	addvar(VAR_VALUE, "vendor", "Regular expression to match UPS Manufacturer string");
+	addvar(VAR_VALUE, "product", "Regular expression to match UPS Product string");
+	addvar(VAR_VALUE, "serial", "Regular expression to match UPS Serial number");
+	addvar(VAR_VALUE, "vendorid", "Regular expression to match UPS Manufacturer numerical ID (4 digits hexadecimal)");
+	addvar(VAR_VALUE, "productid", "Regular expression to match UPS Product numerical ID (4 digits hexadecimal)");
+	addvar(VAR_VALUE, "bus", "Regular expression to match USB bus name");
 }
 
 void upsdrv_banner(void)
@@ -245,8 +442,7 @@ void upsdrv_updateinfo(void)
 	upsdebugx(1, "upsdrv_updateinfo...");
 
 
-	/* FIXME: check for device availability */
-	/* to set datastale! */
+	/* check for device availability to set datastale! */
 	if (hd == NULL)
 	  {
 		upsdebugx(1, "\n=>Got to reconnect!\n");
@@ -259,7 +455,7 @@ void upsdrv_updateinfo(void)
 		 && (data_has_changed != TRUE) )
 	  {
 		/* Wait for HID notifications on Interrupt pipe */
-		if ((evtCount = HIDGetEvents(NULL, eventsList)) > 0)
+		if ((evtCount = HIDGetEvents(udev, NULL, eventsList, subdriver->utab)) > 0)
 		  {
 			upsdebugx(1, "\n=>Got %i HID Objects...", evtCount);
 			
@@ -320,44 +516,30 @@ void upsdrv_updateinfo(void)
 }
 
 
-/* Process status info globally to avoid inconsistencies */
+/* Update ups_status to remember this status item. Interpretation is
+   done in ups_status_set(). */
 static void process_status_info(char *nutvalue)
 {
 	status_lkp_t *status_item;
+	int clear = 0;
 
-	upsdebugx(2, "process_status_info: %s\n", nutvalue);
+	upsdebugx(2, "process_status_info: %s", nutvalue);
 
-	for (status_item = status_info; status_item->status_value != 0 ; status_item++)
+	if (*nutvalue == '!') {
+		nutvalue++;
+		clear = 1;
+	}
+
+	for (status_item = status_info; status_item->status_str != NULL ; status_item++)
 	{
 		if (!strcasecmp(status_item->status_str, nutvalue))
 		{
-			switch (status_item->status_value)
-			{
-				case STATUS_OL: /* clear OB, set OL */
-					ups_status &= ~STATUS_OB;
-					ups_status |= STATUS_OL;
-				break;
-				case STATUS_OB: /* clear OL, set OB */
-					ups_status &= ~STATUS_OL;
-					ups_status |= STATUS_OB;
-				break;
-				case STATUS_LB: /* set LB */ /* FIXME: ?clear OL? */
-					ups_status |= STATUS_LB;   /* else, put in default! */ 
-				break;
-				case STATUS_CHRG: /* clear DISCHRG, set CHRG */
-					ups_status &= ~STATUS_DISCHRG;
-					ups_status |= STATUS_CHRG;
-				break;
-				case STATUS_DISCHRG: /* clear CHRG, set DISCHRG */
-					ups_status &= ~STATUS_CHRG;
-					ups_status |= STATUS_DISCHRG;
-				break;
-				/* FIXME: to be checked */
-				default: /* CAL, TRIM, BOOST, OVER, RB, BYPASS, OFF */
-					ups_status |= status_item->status_value;
-					/* FIXME: When do we reset these? */
-				break;
+			if (clear) {
+				ups_status &= ~status_item->status_mask;
+			} else {
+				ups_status |= status_item->status_mask;
 			}
+			break;
 		}
 	}
 }
@@ -381,34 +563,59 @@ void upsdrv_initinfo(void)
 
 void upsdrv_initups(void)
 {
-	/* Search for the first supported UPS, no matter Mfr or exact product */
-	if ((hd = HIDOpenDevice(device_path, &flg, MODE_OPEN)) == NULL)
-		fatalx("No USB/HID UPS found");
-	else
-		upslogx(1, "Detected an UPS: %s/%s\n", hd->Vendor, hd->Product);
+	char *regex_array[5];
+	int r;
+	int i;
 
-	/* See initinfo for WARNING */
-	switch (hd->VendorID)
-	{
-		case MGE_UPS_SYSTEMS:
-			hid_ups = hid_mge;
-			model_names = mge_models_names;
-		break;
-		case APC:
-			hid_ups = hid_apc;
-			model_names = apc_models_names;
-      			HIDDumpTree(NULL);
-		break;
-		case MUSTEK:
-		case TRIPPLITE:
-		case UNITEK:
-		default:
-			upslogx(1, "Manufacturer not supported!");
-			upslogx(1, "Contact the driver author <arnaud.quette@free.fr / @mgeups.com> with the below information");
-      			HIDDumpTree(NULL);
-			fatalx("Aborting");
-		break;
+        /* process the UPS selection options */
+	regex_array[0] = getval("vendorid");
+	regex_array[1] = getval("productid");
+	regex_array[2] = getval("vendor");
+	regex_array[3] = getval("product");
+	regex_array[4] = getval("serial");
+	regex_array[5] = getval("bus");
+
+	r = new_regex_matcher(&regex_matcher, regex_array, REG_ICASE | REG_EXTENDED);
+	if (r==-1) {
+		fatalx("new_regex_matcher: %s", strerror(errno));
+	} else if (r) {
+		fatalx("invalid regular expression: %s", regex_array[r]);
 	}
+	/* link the matchers */
+	regex_matcher->next = subdriver_matcher;
+
+	/* Search for the first supported UPS matching the regular
+	   expression */
+	if ((hd = HIDOpenDevice(&udev, &curDevice, regex_matcher, MODE_OPEN)) == NULL)
+		fatalx("No matching USB/HID UPS found");
+	else
+		upslogx(1, "Detected a UPS: %s/%s", hd->Vendor ? hd->Vendor : "unknown", hd->Product ? hd->Product : "unknown");
+
+	/* create a new matcher for later reopening */
+	reopen_matcher = new_exact_matcher(hd);
+	if (!reopen_matcher) {
+		upsdebugx(2, "new_exact_matcher: %s", strerror(errno));
+	}
+	/* link the two matchers */
+	reopen_matcher->next = regex_matcher;
+
+	/* select the subdriver for this device */
+	for (i=0; subdriver_list[i] != NULL; i++) {
+		if (subdriver_list[i]->claim(hd)) {
+			break;
+		}
+	}
+	subdriver = subdriver_list[i];
+	if (!subdriver) {
+		upslogx(1, "Manufacturer not supported!");
+		upslogx(1, "Contact the driver author <arnaud.quette@free.fr / @mgeups.com> with the below information");
+		HIDDumpTree(udev, subdriver->utab);
+		fatalx("Aborting");
+	}
+
+	upslogx(2, "Using subdriver: %s", subdriver->name);
+
+	HIDDumpTree(udev, subdriver->utab);
 
 	/* init polling frequency */
 	if ( getval(HU_VAR_POLLFREQ) )
@@ -417,72 +624,43 @@ void upsdrv_initups(void)
 
 void upsdrv_cleanup(void)
 {
-	if (hd != NULL)
-		HIDCloseDevice(hd);
+	if (hd != NULL) {
+		HIDCloseDevice(udev);
+		udev = NULL;
+	}
 }
 
 /**********************************************************************
  * Support functions
  *********************************************************************/
 
-void identify_ups ()
+static void identify_ups ()
 {
-	char *string;
-	char *ptr1, *ptr2, *str;
-	char *finalname = NULL;
-	float appPower;
+	char *model;
+	char *mfr;
+	char *serial;
 
 	upsdebugx (2, "entering identify_ups(0x%04x, 0x%04x)\n", 
-			   hd->VendorID, /*FIXME: iManufacturer? */
-			   hd->iProduct);
+			   hd->VendorID,
+			   hd->ProductID);
 
-	switch (hd->VendorID)
-	{
-		case MGE_UPS_SYSTEMS:
-			/* Get iModel and iProduct strings */
-			if ((string = HIDGetItemString("UPS.PowerSummary.iModel")) != NULL)
-				finalname = get_model_name(hd->Product, string);
-			else
-			{
-				/* Try with ConfigApparentPower */
-				if (HIDGetItemValue("UPS.Flow.[4].ConfigApparentPower", &appPower) != 0 )
-				{
-					string = xmalloc(16);
-					sprintf(string, "%i", (int)appPower);
-					finalname = get_model_name(hd->Product, string);
-					free (string);
-				}
-				else
-				finalname = hd->Product;
-			}
-		break;
-		case APC:
-			/* FIXME?: what is the path "UPS.APC_UPS_FirmwareRevision"? */
-			str = hd->Product;
-			ptr1 = strstr(str, "FW:");
-			if (ptr1)
-			{
-				*(ptr1 - 1) = '\0';
-				ptr1 += strlen("FW:");
-				ptr2 = strstr(ptr1, "USB FW:");
-				if (ptr2)
-				{
-					*(ptr2 - 1) = '\0';
-					ptr2 += strlen("USB FW:");
-					dstate_setinfo("ups.firmware.aux", "%s", ptr2);
-				}
-				dstate_setinfo("ups.firmware", "%s", ptr1);
-			}
-			finalname = str;
-			break;
-		default: /* Nothing to do */
-		break;
+	/* use vendor-specific method for calculating human-readable
+	   manufacturer, model, and serial strings */
+
+	model = subdriver->format_model(hd);
+	mfr = subdriver->format_mfr(hd);
+	serial = subdriver->format_serial(hd);
+
+	/* set corresponding variables */
+	if (mfr != NULL) {
+		dstate_setinfo("ups.mfr", "%s", mfr);
+	}
+	if (model != NULL) {
+		dstate_setinfo("ups.model", "%s", model);
+	}
+	if (serial != NULL) {
+		dstate_setinfo("ups.serial", "%s", serial);
 	}
-
-	/* Actual information setting */
-	dstate_setinfo("ups.mfr", "%s", hd->Vendor);
-	dstate_setinfo("ups.model", "%s", finalname);
-	dstate_setinfo("ups.serial", "%s", (hd->Serial != NULL)?hd->Serial:"unknown");
 }
 
 /* walk ups variables and set elements of the info array. */
@@ -496,7 +674,7 @@ static bool hid_ups_walk(int mode)
 	/* 3 modes: HU_WALKMODE_INIT, HU_WALKMODE_QUICK_UPDATE and HU_WALKMODE_FULL_UPDATE */
 	
 	/* Device data walk ----------------------------- */
-	for ( item = hid_ups ; item->info_type != NULL ; item++ )
+	for ( item = subdriver->hid2nut ; item->info_type != NULL ; item++ )
 	  {
 		/* Check if we are asked to stop (reactivity++) */
 		if (exit_flag != 0)
@@ -520,7 +698,7 @@ static bool hid_ups_walk(int mode)
 			  /* Check instant commands availability */
 			  if (item->hidflags & HU_TYPE_CMD)
 				{
-				  if (HIDGetItemValue(item->hidpath, &value) == 1 )
+				  if (HIDGetItemValue(udev, item->hidpath, &value, subdriver->utab) == 1 )
 					dstate_addcmd(item->info_type);
 
 				  continue;
@@ -531,7 +709,7 @@ static bool hid_ups_walk(int mode)
 				  /* Check if exists (if necessary) before creation */
 				  if (item->hidpath != NULL)
 					{
-					  if ((retcode = HIDGetItemValue(item->hidpath, &value)) != 1 )
+					  if ((retcode = HIDGetItemValue(udev, item->hidpath, &value, subdriver->utab)) != 1 )
 						continue;
 					}
 				  else
@@ -582,7 +760,7 @@ static bool hid_ups_walk(int mode)
 			 && !(item->hidflags & HU_FLAG_OK) )
 		  continue;
 
-		if ((retcode = HIDGetItemValue(item->hidpath, &value)) == 1 )
+		if ((retcode = HIDGetItemValue(udev, item->hidpath, &value, subdriver->utab)) == 1 )
 		  {
 			/* deal with status items */
 			if (!strncmp(item->info_type, "ups.status", 10))
@@ -609,7 +787,6 @@ static bool hid_ups_walk(int mode)
 				
 				if (mode == HU_WALKMODE_INIT)
 				  {
-					/* TODO: verify setability/RW with (hData.Attribute != ATTR_DATA_CST) */
 					dstate_setflags(item->info_type, item->info_flags);
 				  }
 			  }
@@ -653,7 +830,7 @@ static bool hid_ups_walk(int mode)
 	/* Reserved values: -1/-10 for nul delay, -2 can't get value */
 	/* device has been disconnected, try to reconnect */
 	if ( (retcode == -EPERM) || (retcode == -EPIPE)
-		 || (retcode == -ENODEV) || (retcode == -EACCES) )
+	     || (retcode == -ENODEV) || (retcode == -EACCES))
 	  {
 		hd = NULL;
 		reconnect_ups();
@@ -675,43 +852,67 @@ static void reconnect_ups(void)
 	  upsdebugx(2, "==================================================");
 	  
 	  /* Not really useful as the device is no more reachable */
-	  HIDCloseDevice(NULL);
+	  HIDCloseDevice(udev);
+	  udev = NULL;
 	  
-	  if ((hd = HIDOpenDevice(device_path, &flg, MODE_REOPEN)) == NULL)
+	  if ((hd = HIDOpenDevice(&udev, &curDevice, reopen_matcher, MODE_REOPEN)) == NULL)
 		dstate_datastale();
 	}
 }
 
-/* Process the whole ups.status */
+/* Convert the local status information to NUT format and set NUT
+   status. */
 static void ups_status_set(void)
 {
 	/* clear status buffer before begining */
 	status_init();
   
-	if (ups_status & STATUS_CAL)
-	  status_set("CAL");		/* calibration */
-	if (ups_status & STATUS_TRIM)
-	  status_set("TRIM");		/* SmartTrim */
-	if (ups_status & STATUS_BOOST)
-	  status_set("BOOST");	/* SmartBoost */
-	if (ups_status & STATUS_OL)
-	  status_set("OL");		/* on line */
-	if (ups_status & STATUS_OB)
-	  status_set("OB");		/* on battery */
-	if (ups_status & STATUS_OVER)
-	  status_set("OVER");		/* overload */
-	if (ups_status & STATUS_LB)
-	  status_set("LB");		/* low battery */
-	if (ups_status & STATUS_RB)
-	  status_set("RB");		/* replace batt */
-	if (ups_status & STATUS_BYPASS)
-	  status_set("BYPASS");	/* on bypass */   
-	if (ups_status & STATUS_CHRG)
-	  status_set("CHRG");		/* charging */
-	if (ups_status & STATUS_DISCHRG)
-	  status_set("DISCHRG");	/* discharging */         
-	if ( (ups_status & STATUS_OFF) || (ups_status == 0) )
-	  status_set("OFF");
+	if (ups_status & STATUS_ONLINE) {
+		status_set("OL");		/* on line */
+	} else {
+		status_set("OB");               /* on battery */
+	}
+	if (ups_status & STATUS_DISCHRG) {
+		status_set("DISCHRG");	        /* discharging */         
+	}
+	if (ups_status & STATUS_CHRG) {
+		status_set("CHRG");		/* charging */
+	}
+	if (ups_status & (STATUS_LOWBATT
+			  | STATUS_SHUTDOWNIMM
+			  | STATUS_TIMELIMITEXP)) {
+		status_set("LB");		/* low battery */
+	}
+	if (ups_status & STATUS_OVERLOAD) {
+		status_set("OVER");		/* overload */
+	}
+	if (ups_status & STATUS_REPLACEBATT) {
+		status_set("RB");		/* replace batt */
+	}
+	if (ups_status & STATUS_TRIM) {
+		status_set("TRIM");		/* SmartTrim */
+	}
+	if (ups_status & STATUS_BOOST) {
+		status_set("BOOST");	        /* SmartBoost */
+	}
+	if (ups_status & STATUS_BYPASS) {
+		status_set("BYPASS");	        /* on bypass */   
+	}
+	if (ups_status & STATUS_OFF) {
+		status_set("OFF");              /* ups is off */
+	}
+	if (ups_status & STATUS_CAL) {
+		status_set("CAL");		/* calibration */
+	}
+	if (ups_status & STATUS_OVERHEAT) {
+		status_set("OVERHEAT");		/* overheat; Belkin */
+	}
+	if (ups_status & STATUS_COMMFAULT) {
+		status_set("COMMFAULT");	/* UPS fault; Belkin */
+	}
+	if (ups_status & STATUS_DEPLETED) {
+		status_set("DEPLETED");		/* battery depleted; Belkin */
+	}
 	
 	/* Commit the status buffer */
 	status_commit();
@@ -724,7 +925,7 @@ static hid_info_t *find_nut_info(const char *varname)
 {
   hid_info_t *hidups_item;
 
-  for (hidups_item = hid_ups; hidups_item->info_type != NULL ; hidups_item++) {
+  for (hidups_item = subdriver->hid2nut; hidups_item->info_type != NULL ; hidups_item++) {
     if (!strcasecmp(hidups_item->info_type, varname))
       return hidups_item;
   }
@@ -733,6 +934,26 @@ static hid_info_t *find_nut_info(const char *varname)
   return NULL;
 }
 
+/* find info element definition in info array by NUT varname. Only
+ * return items whose HID path actually exists.  By this, we enable
+ * multiple alternative definitions of an instant command; the first
+ * one that works for *this* UPS will be used. 
+ */
+static hid_info_t *find_nut_info_valid(const char *varname)
+{
+  hid_info_t *hidups_item;
+  float value;
+
+  for (hidups_item = subdriver->hid2nut; hidups_item->info_type != NULL ; hidups_item++) {
+    if (!strcasecmp(hidups_item->info_type, varname))
+      if (HIDGetItemValue(udev, hidups_item->hidpath, &value, subdriver->utab) == 1)
+	return hidups_item;
+  }
+
+  upsdebugx(2, "find_nut_info: unknown info type: %s\n", varname);
+  return NULL;
+}
+
 /* find info element definition in info array
  * by HID varname.
  */
@@ -740,7 +961,7 @@ static hid_info_t *find_hid_info(const char *hidname)
 {
   hid_info_t *hidups_item;
 
-  for (hidups_item = hid_ups; hidups_item->info_type != NULL ; hidups_item++) {
+  for (hidups_item = subdriver->hid2nut; hidups_item->info_type != NULL ; hidups_item++) {
 
 	/* Skip NULL HID path (server side vars) */
 	if (hidups_item->hidpath == NULL)
@@ -754,9 +975,10 @@ static hid_info_t *find_hid_info(const char *hidname)
   return NULL;
 }
 
+#if 0  /* NOT USED */
 /* find the HID Item value matching that NUT value */
 /* useful for set with value lookup... */
-/* static long hu_find_valinfo(info_lkp_t *hid2info, char* value)
+static long hu_find_valinfo(info_lkp_t *hid2info, char* value)
 {
   info_lkp_t *info_lkp;
   
@@ -772,15 +994,24 @@ static hid_info_t *find_hid_info(const char *hidname)
   }
   upsdebugx(3, "hu_find_valinfo: no matching HID value for this INFO_* value (%s)", value);
   return -1;
-} */
+}
+#endif  /* NOT USED */
 
 /* find the NUT value matching that HID Item value */
 static char *hu_find_infoval(info_lkp_t *hid2info, long value)
 {
   info_lkp_t *info_lkp;
+  char *nut_value;
   
   upsdebugx(3, "hu_find_infoval: searching for value = %ld\n", value);
   
+  if (hid2info->fun != NULL) {
+    nut_value = hid2info->fun(value);
+    upsdebugx(3, "hu_find_infoval: found %s (value: %ld)\n",
+	      nut_value, value);
+    return nut_value;
+  }
+
   for (info_lkp = hid2info; (info_lkp != NULL) &&
 	 (strcmp(info_lkp->nut_value, "NULL")); info_lkp++) {
     
@@ -795,28 +1026,3 @@ static char *hu_find_infoval(info_lkp_t *hid2info, long value)
   return NULL;
 }
 
-/* All the logic for formatting finely the UPS model name */
-char *get_model_name(char *iProduct, char *iModel)
-{
-  models_name_t *model = NULL;
-
-  upsdebugx(2, "get_model_name(%s, %s)\n", iProduct, iModel);
-
-  /* Search for formatting rules */
-  for ( model = model_names ; model->iProduct != NULL ; model++ )
-	{
-	  upsdebugx(2, "comparing with: %s", model->finalname);
-	  /* FIXME: use comp_size if not -1 */
-	  if ( (!strncmp(iProduct, model->iProduct, strlen(model->iProduct)))
-		   && (!strncmp(iModel, model->iModel, strlen(model->iModel))) )
-		{
-		  upsdebugx(2, "Found %s\n", model->finalname);
-		  break;
-		}
-	}
-  /* FIXME: if we end up with model->iProduct == NULL
-   * then process name in a generic way (not yet supported models!)
-   */
-  return model->finalname;
-}
-
