@@ -1,45 +1,39 @@
 /* newhidups.c - Driver for serial/USB HID UPS units
  *
- * Copyright (C) 2003 - 2005
- *   Arnaud Quette <arnaud.quette@free.fr> && <arnaud.quette@mgeups.com>
+ * Copyright (C)
+ *  2003 / 2005 - Arnaud Quette <http://arnaud.quette.free.fr/contact.html>
  *
  * This program is sponsored by MGE UPS SYSTEMS - opensource.mgeups.com
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or 
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  */
 
+#ifndef NEWHIDUPS_H
+#define NEWHIDUPS_H
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include "config.h"
+#include "libusb.h"
 
-#define DRIVER_VERSION		"0.23"
-
-/* --------------------------------------------------------------- */
-/*      Supported Manufacturers IDs                                */
-/* --------------------------------------------------------------- */
-
-#define MGE_UPS_SYSTEMS		0x0463		/* All models */
-#define APC			0x051d		/* All models */
-/* Unsupported! (need spec/hardware/help) */
-#define MUSTEK			0x0665		/* models: 0x5161... */
-#define TRIPPLITE		0x09ae		/* models IDs? */
-#define UNITEK			0x0F03		/* models: 0x0001... */
+#define DRIVER_VERSION		"0.28"
 
+extern usb_dev_handle *udev;
 
 /* --------------------------------------------------------------- */
 /*      Model Name formating entries                               */
@@ -62,7 +56,7 @@ typedef struct
 #define DEFAULT_ONDELAY		30	/* Delay between return of utility power */
 					/* and powering up of load, in seconds */
 					/* CAUTION: ondelay > offdelay */
-#define DEFAULT_OFFDELAY	20	/* Delay before power off, in seconds */ 
+#define DEFAULT_OFFDELAY	20	/* Delay before power off, in seconds */
 #define DEFAULT_POLLFREQ	30	/* Polling interval, in seconds */
 					/* The driver will wait for Interrupt */
 					/* and do "light poll" in the meantime */
@@ -80,40 +74,33 @@ typedef struct
 /* Struct & data for ups.status processing                         */
 /* --------------------------------------------------------------- */
 
+/* Note: this structure holds internal status info, directly as
+   collected from the hardware; not yet converted to official NUT
+   status */
 typedef struct {
 	char	*status_str;	/* ups.status string */
-	int	status_value;	/* ups.status value */
+	int	status_mask;	/* ups_status mask */
 } status_lkp_t;
 
-#define STATUS_CAL		1       /* calibration */
-#define STATUS_TRIM		2       /* SmartTrim */
-#define STATUS_BOOST		4       /* SmartBoost */
-#define STATUS_OL		8       /* on line */
-#define STATUS_OB		16      /* on battery */
-#define STATUS_OVER		32      /* overload */
-#define STATUS_LB		64      /* low battery */
-#define STATUS_RB		128     /* replace battery */
-#define STATUS_BYPASS		256	/* on bypass */
-#define STATUS_OFF		512	/* ups is off */
-#define STATUS_CHRG		1024	/* charging */
-#define STATUS_DISCHRG		2048	/* discharging */
-
-status_lkp_t status_info[] = {
-  { "CAL", STATUS_CAL },
-  { "TRIM", STATUS_TRIM },
-  { "BOOST", STATUS_BOOST },
-  { "OL", STATUS_OL },
-  { "OB", STATUS_OB },
-  { "OVER", STATUS_OVER },
-  { "LB", STATUS_LB },
-  { "RB", STATUS_RB },
-  { "BYPASS", STATUS_BYPASS },
-  { "OFF", STATUS_OFF },
-  { "CHRG", STATUS_CHRG },
-  { "DISCHRG", STATUS_DISCHRG },
-  { "NULL", 0 },
-};
+#define STATUS_ONLINE           0x00001  /* on line */
+#define STATUS_DISCHRG          0x00002  /* discharging */
+#define STATUS_CHRG             0x00004  /* charging */
+#define STATUS_LOWBATT		0x00008  /* low battery */
+#define STATUS_OVERLOAD		0x00010  /* overload */
+#define STATUS_REPLACEBATT	0x00020  /* replace battery */
+#define STATUS_SHUTDOWNIMM	0x00040  /* shutdown imminent */
+#define STATUS_TRIM		0x00080  /* SmartTrim */
+#define STATUS_BOOST		0x00100  /* SmartBoost */
+#define STATUS_BYPASS		0x00200  /* on bypass */
+#define STATUS_OFF		0x00400  /* ups is off */
+#define STATUS_CAL 		0x00800  /* calibration */
+#define STATUS_OVERHEAT         0x01000  /* overheat; Belkin */
+#define STATUS_COMMFAULT        0x02000  /* UPS fault; Belkin */
+#define STATUS_DEPLETED         0x04000  /* battery depleted; Belkin */
+#define STATUS_TIMELIMITEXP     0x08000  /* time limit expired; APC */
+#define STATUS_BATTERYPRES      0x10000  /* battery present; APC */
 
+extern status_lkp_t status_info[];
 
 /* --------------------------------------------------------------- */
 /* Struct & data for lookup between HID and NUT values             */
@@ -123,73 +110,33 @@ status_lkp_t status_info[] = {
 typedef struct {
 	long	hid_value;	/* HID value */
 	char	*nut_value;	/* NUT value */
+        char    *(*fun)(long value); /* special case: if fun!=NULL, then
+				     ignore hid_value and nut_value,
+				     and use the conversion function
+				     instead. This is used for more
+				     complex formatting such as
+				     dates. Fun is expected to return
+				     a statically allocated string. */
 } info_lkp_t;
 
-/* Actual value lookup tables => should be fine for all Mfrs (TODO: validate it!) */
-info_lkp_t onbatt_info[] = {
-  { 0, "OB" },
-  { 1, "OL" },
-  { 0, "NULL" }
-};
-info_lkp_t discharging_info[] = {
-  { 1, "DISCHRG" },
-  { 0, "NULL" }
-};
-info_lkp_t charging_info[] = {
-  { 1, "CHRG" },
-  { 0, "NULL" }
-};
-info_lkp_t lowbatt_info[] = {
-  { 1, "LB" },
-  { 0, "NULL" }
-};
-info_lkp_t overbatt_info[] = {
-  { 1, "OVER" },
-  { 0, "NULL" }
-};
-info_lkp_t replacebatt_info[] = {
-  { 1, "RB" },
-  { 0, "NULL" }
-};
-info_lkp_t shutdownimm_info[] = {
-  { 1, "LB" },
-  { 0, "NULL" }
-};
-info_lkp_t trim_info[] = {
-  { 1, "TRIM" },
-  { 0, "NULL" }
-};
-info_lkp_t boost_info[] = {
-  { 1, "BOOST" },
-  { 0, "NULL" }
-};
-/* FIXME: extend ups.status for BYPASS Manual/Automatic */
-info_lkp_t bypass_info[] = {
-  { 1, "BYPASS" },
-  { 0, "NULL" }
-};
-info_lkp_t off_info[] = {
-  { 0, "OFF" },
-  { 0, "NULL" }
-};
-/* FIXME: add CAL */
-
-info_lkp_t test_write_info[] = {
-  { 0, "No test" },
-  { 1, "Quick test" },
-  { 2, "Deep test" },
-  { 3, "Abort test" },
-  { 0, "NULL" }
-};
-info_lkp_t test_read_info[] = {
-  { 1, "Done and passed" },
-  { 2, "Done and warning" },
-  { 3, "Done and error" },
-  { 4, "Aborted" },
-  { 5, "In progress" },
-  { 6, "No test initiated" },
-  { 0, "NULL" }
-};
+/* declarations of public lookup tables */
+extern info_lkp_t online_info[];
+extern info_lkp_t discharging_info[];
+extern info_lkp_t charging_info[];
+extern info_lkp_t lowbatt_info[];
+extern info_lkp_t overload_info[];
+extern info_lkp_t replacebatt_info[];
+extern info_lkp_t shutdownimm_info[];
+extern info_lkp_t trim_info[];
+extern info_lkp_t boost_info[];
+extern info_lkp_t bypass_info[];
+extern info_lkp_t off_info[];
+extern info_lkp_t test_write_info[];
+extern info_lkp_t test_read_info[];
+extern info_lkp_t beeper_info[];
+extern info_lkp_t date_conversion[];
+extern info_lkp_t hex_conversion[];
+extern info_lkp_t stringid_conversion[];
 
 /* --------------------------------------------------------------- */
 /* Structure containing information about how to get/set data      */
@@ -212,7 +159,6 @@ typedef struct {
 /*	void *next;			*//* next hid_info_t */
 } hid_info_t;
 
-
 /* Data walk modes */
 #define HU_WALKMODE_INIT		1
 #define HU_WALKMODE_QUICK_UPDATE	2
@@ -224,7 +170,7 @@ typedef struct {
 #define HU_FLAG_SEMI_STATIC		4		/* retrieve info smartly */
 #define HU_FLAG_ABSENT			8		/* data is absent in the device, */
 							/* use default value. */
-#define HU_FLAG_QUICK_POLL		16		/* Mandatory vars	*/		
+#define HU_FLAG_QUICK_POLL		16		/* Mandatory vars	*/
 #define HU_FLAG_STALE			32		/* data stale, don't try too often. */
 
 /* hints for su_ups_set, applicable only to rw vars */
@@ -236,4 +182,33 @@ typedef struct {
 
 #define MAX_TRY		2		/* max number of GetItem retry */
 
-/* TODO: create an Mfr table (int VendorID, hid_info_t *hid_mfr, ...) */
+/* --------------------------------------------------------------- */
+/*      Subdriver interface                                        */
+/* --------------------------------------------------------------- */
+
+/* subdriver structure. Each subdriver is intended to support a
+ * particular manufacturer (e.g. MGE, APC, Belkin), or a particular
+ * range of models. */
+
+struct subdriver_s {
+	char *name;                  /* name of this subdriver */
+	int (*claim)(HIDDevice *hd); /* return 1 if device covered by
+				      * this subdriver */
+	usage_tables_t *utab;        /* points to array of usage tables */
+	hid_info_t *hid2nut;         /* main table of vars and instcmds */
+	int (*shutdown)(int ondelay, int offdelay);
+                                     /* driver-specific shutdown cmd.
+					Returns 1 on success, 0 on
+					failure */
+	char *(*format_model)(HIDDevice *hd);  /* driver-specific methods */
+	char *(*format_mfr)(HIDDevice *hd);    /* for preparing human-    */
+	char *(*format_serial)(HIDDevice *hd); /* readable information    */
+};
+typedef struct subdriver_s subdriver_t;
+
+
+/* the following functions are exported for the benefit of subdrivers */
+int instcmd(const char *cmdname, const char *extradata);
+int setvar(const char *varname, const char *val);
+
+#endif /* NEWHIDUPS_H */
