@@ -124,12 +124,13 @@
 
 
 #include "main.h"
+#include "serial.h"
 
-#define DRV_VERSION	"0.01"
+#define DRV_VERSION	"0.02"
 
 #define MAX_RESPONSE_LENGTH 256
 
-static char *test_result_names[] = {
+static const char *test_result_names[] = {
 	"No test performed",
 	"Passed",
 	"In progress",
@@ -141,7 +142,7 @@ static char *test_result_names[] = {
 
 static struct {
 	int code;
-	char *name;
+	const char *name;
 } sensitivity[] = {
 	{0, "Normal"},
 	{1, "Reduced"},
@@ -214,19 +215,19 @@ static int do_command(char type, const char *command,
 	char *ptr;
 
 	count = 0;
-	upsflushin(0, nut_debug_level, "");
+	ser_flush_in(upsfd, "", nut_debug_level);
 
-	if (upssend("~00%c%03d%s%s", type,
+	if (ser_send(upsfd, "~00%c%03d%s%s", type,
 	            strlen(command) + strlen(parameters), command,
 	            parameters) <= 0)
 		return -1;
 
-	if (upsrecvchars(buffer, 4) <= 0)
+	if (ser_get_buf_len(upsfd, buffer, 4, 3, 0) <= 0)
 		return -1;
 	buffer[4] = '\0';
 
 	if (!strcmp(buffer, "~00D")) {
-		if (upsrecvchars(buffer, 3) <= 0)
+		if (ser_get_buf_len(upsfd, buffer, 4, 3, 0) <= 0)
 			return -1;
 		buffer[3] = '\0';
 		count = atoi(buffer);
@@ -238,7 +239,7 @@ static int do_command(char type, const char *command,
 			return 0;
 		}
 
-		if (upsrecvchars(response, count) <= 0)
+		if (ser_get_buf_len(upsfd, response, count, 3, 0) <= 0)
 			return -1;
 		response[count] = '\0';
 		/* Tripp Lite pads their string responses with spaces.
@@ -249,25 +250,25 @@ static int do_command(char type, const char *command,
 		while (ptr > response && *ptr == ' ')
 			ptr--;
 		*(ptr + 1) = '\0';
-		nolongertimeout();
+		ser_comm_good();
 		return count;
 	}
 	if (!strcmp(buffer, "~00A")) {
 		if (response)
 			*response = '\0';
-		nolongertimeout();
+		ser_comm_good();
 		return count;
 	}
 	if (!strcmp(buffer, "~00R"))
-		nolongertimeout();
+		ser_comm_good();
 
 	return -1;
 }
 
-static char *field(char *str, int field)
+static char *field(char *str, int fieldnum)
 {
 
-	while (str && field--) {
+	while (str && fieldnum--) {
 		str = strchr(str, ';');
 		if (str)
 			str++;
@@ -367,7 +368,7 @@ static void set_transfer_voltage_high(int val) {
 
 static int get_sensitivity(void) {
 	char response[MAX_RESPONSE_LENGTH];
-	int i;
+	unsigned int i;
 
 	if (do_command(POLL, VOLTAGE_SENSITIVITY, "", response) <= 0)
 		return 0;
@@ -384,7 +385,7 @@ static int get_sensitivity(void) {
 
 static void set_sensitivity(const char *val) {
 	char parm[20];
-	int i;
+	unsigned int i;
 
 	for (i = 0; i < sizeof(sensitivity) / sizeof(sensitivity[0]); i++) {
 		if (!strcasecmp(val, sensitivity[i].name)) {
@@ -526,9 +527,9 @@ static int init_comm(void)
 void upsdrv_initinfo(void)
 {
 	char response[MAX_RESPONSE_LENGTH];
-	int min_low_transfer, max_low_transfer;
-	int min_high_transfer, max_high_transfer;
-	int i;
+	unsigned int min_low_transfer, max_low_transfer;
+	unsigned int min_high_transfer, max_high_transfer;
+	unsigned int i;
 	char *ptr;
 
 	if (!init_comm())
@@ -615,8 +616,8 @@ void upsdrv_initinfo(void)
 	/* add all the variables that change regularly */
 	upsdrv_updateinfo();
 
-	upsh.new_instcmd = instcmd;
-	upsh.new_setvar = setvar;
+	upsh.instcmd = instcmd;
+	upsh.setvar = setvar;
 
 	printf("Detected %s %s on %s\n", dstate_getinfo("ups.mfr"),
 	       dstate_getinfo("ups.model"), device_path);
@@ -740,11 +741,17 @@ void upsdrv_updateinfo(void)
 	}
 
 	if (do_command(POLL, TEST_RESULT, "", response) > 0) {
-		i = atoi(response);
-		if (i < 0 || i >= sizeof(test_result_names) /
-		                  sizeof(test_result_names[0]))
-			i = 0;
-		dstate_setinfo("ups.test.result", "%s", test_result_names[i]);
+		int	r;
+		size_t	trsize;
+
+		r = atoi(response);
+		trsize = sizeof(test_result_names) / 
+			sizeof(test_result_names[0]);
+
+		if ((r < 0) || (r >= (int) trsize))
+			r = 0;
+
+		dstate_setinfo("ups.test.result", "%s", test_result_names[r]);
 	}
 
 	if (do_command(POLL, ENVIRONMENT_INFORMATION, "", response) > 0) {
@@ -808,32 +815,11 @@ void upsdrv_banner(void)
 
 void upsdrv_initups(void)
 {
-
-	open_serial(device_path, B2400);
-	/* to get variables and flags from the command line, use this:
-	 *
-	 * first populate with upsdrv_buildvartable above, then...
-	 *
-	 *                   set flag foo : /bin/driver -x foo
-	 * set variable 'cable' to '1234' : /bin/driver -x cable=1234
-	 *
-	 * to test flag foo in your code:
-	 *
-	 * 	if (testvar("foo"))
-	 * 		do_something();
-	 *
-	 * to show the value of cable:
-	 *
-	 *      if ((cable == getval("cable")))
-	 *		printf("cable is set to %s\n", getval("cable"));
-	 *	else
-	 *		printf("cable is not set!\n");
-	 *
-	 * don't use NULL pointers - test the return result first!
-	 */
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B2400);
 }
 
 void upsdrv_cleanup(void)
 {
-	/* free(dynamic_mem); */
+	ser_close(upsfd, device_path);
 }
